@@ -41,7 +41,7 @@
 #include <asm/irq.h>
 #include <linux/leds.h>
 #include <asm/mach-types.h>
-#include <asm/sched_clock.h>
+#include <linux/sched_clock.h>
 
 #include <asm/mach/arch.h>
 #include <asm/mach/flash.h>
@@ -83,6 +83,8 @@
 /* command line parameters */
 static unsigned boardrev, serial;
 static unsigned uart_clock;
+static unsigned disk_led_gpio = 16;
+static unsigned disk_led_active_low = 1;
 static unsigned reboot_part = 0;
 
 static void __init bcm2708_init_led(void);
@@ -641,29 +643,131 @@ static struct platform_device bcm2708_i2s_device = {
 
 #if defined(CONFIG_SND_BCM2708_SOC_HIFIBERRY_DAC) || defined(CONFIG_SND_BCM2708_SOC_HIFIBERRY_DAC_MODULE)
 static struct platform_device snd_hifiberry_dac_device = {
-        .name = "snd-hifiberry-dac",
+	.name = "snd-hifiberry-dac",
+	.id = 0,
+	.num_resources = 0,
+};
+
+static struct platform_device snd_pcm5102a_codec_device = {
+	.name = "pcm5102a-codec",
+	.id = -1,
+	.num_resources = 0,
+};
+#endif
+
+#if defined(CONFIG_SND_BCM2708_SOC_HIFIBERRY_DIGI) || defined(CONFIG_SND_BCM2708_SOC_HIFIBERRY_DIGI_MODULE)
+static struct platform_device snd_hifiberry_digi_device = {
+        .name = "snd-hifiberry-digi",
         .id = 0,
         .num_resources = 0,
 };
 
-static struct platform_device snd_pcm5102a_codec_device = {
-        .name = "pcm5102a-codec",
-        .id = -1,
-        .num_resources = 0,
+static struct i2c_board_info __initdata snd_wm8804_i2c_devices[] = {
+        {
+                I2C_BOARD_INFO("wm8804", 0x3b)
+        },
 };
+
 #endif
 
 #if defined(CONFIG_SND_BCM2708_SOC_RPI_DAC) || defined(CONFIG_SND_BCM2708_SOC_RPI_DAC_MODULE)
 static struct platform_device snd_rpi_dac_device = {
-        .name = "snd-rpi-dac",
-        .id = 0,
-        .num_resources = 0,
+	.name = "snd-rpi-dac",
+	.id = 0,
+	.num_resources = 0,
 };
 
 static struct platform_device snd_pcm1794a_codec_device = {
-        .name = "pcm1794a-codec",
-        .id = -1,
-        .num_resources = 0,
+	.name = "pcm1794a-codec",
+	.id = -1,
+	.num_resources = 0,
+};
+#endif
+
+#ifdef CONFIG_SND_BCM2708_SOC_RPI_CODEC_MBED_MODULE
+static struct platform_device snd_rpi_mbed_device = {
+	.name = "snd-rpi-mbed",
+	.id = 0,
+	.num_resources = 0,
+};
+
+
+static struct i2c_board_info __initdata snd_rpi_mbed_i2c_devices[] = {
+	{
+		I2C_BOARD_INFO("tlv320aic23", 0x1b)
+	},
+};
+#endif
+
+
+#ifdef CONFIG_SND_BCM2708_SOC_RPI_CODEC_TDA1541A_MODULE
+static struct platform_device snd_rpi_tda1541a_device = {
+	.name = "snd-rpi-tda1541a",
+	.id = 0,
+	.num_resources = 0,
+};
+
+static struct platform_device snd_rpi_tda1541a_codec_device = {
+	.name = "tda1541a-codec",
+	.id = -1,
+	.num_resources = 0,
+};
+#endif
+
+#ifdef CONFIG_SND_BCM2708_SOC_RPI_CODEC_PROTO_MODULE
+static struct platform_device snd_rpi_proto_device = {
+	.name = "snd-rpi-proto",
+	.id = 0,
+	.num_resources = 0,
+};
+
+static struct i2c_board_info __initdata snd_rpi_proto_i2c_devices[] = {
+	{
+		I2C_BOARD_INFO("wm8731", 0x1a)
+	},
+};
+#endif
+
+#ifdef CONFIG_SND_BCM2708_SOC_RPI_CODEC_CS534X_MODULE
+static struct platform_device snd_rpi_cs534x_device = {
+	.name = "snd-rpi-cs534x",
+	.id = 0,
+	.num_resources = 0,
+};
+
+static struct platform_device snd_rpi_cs534x_codec_device = {
+	.name = "cs534x-codec",
+	.id = -1,
+	.num_resources = 0,
+};
+
+#endif
+
+#ifdef CONFIG_SND_BCM2708_SOC_RPI_CODEC_ESS9018_MODULE
+static struct platform_device snd_rpi_ess9018_device = {
+	.name = "snd-rpi-ess9018",
+	.id = 0,
+	.num_resources = 0,
+};
+
+static struct platform_device snd_rpi_ess9018_codec_device = {
+	.name = "ess9018-codec",
+	.id = -1,
+	.num_resources = 0,
+};
+#endif
+
+#ifdef CONFIG_SND_BCM2708_SOC_RPI_CODEC_PCM5102A_MODULE
+static struct platform_device snd_rpi_pcm5102a_device = {
+	.name = "snd-rpi-pcm5102a",
+	.id = 0,
+	.num_resources = 0,
+};
+
+static struct platform_device snd_rpi_pcm5102a_codec_device = {
+	.name = "pcm5102a-codec",
+	.id = -1,
+	.num_resources = 0,
 };
 #endif
 
@@ -798,13 +902,49 @@ void __init bcm2708_init(void)
 #endif
 
 #if defined(CONFIG_SND_BCM2708_SOC_HIFIBERRY_DAC) || defined(CONFIG_SND_BCM2708_SOC_HIFIBERRY_DAC_MODULE)
-        bcm_register_device(&snd_hifiberry_dac_device);
-        bcm_register_device(&snd_pcm5102a_codec_device);
+	bcm_register_device(&snd_hifiberry_dac_device);
+	bcm_register_device(&snd_pcm5102a_codec_device);
+#endif
+
+#if defined(CONFIG_SND_BCM2708_SOC_HIFIBERRY_DIGI) || defined(CONFIG_SND_BCM2708_SOC_HIFIBERRY_DIGI_MODULE)
+        bcm_register_device(&snd_hifiberry_digi_device);
+        i2c_register_board_info(1, snd_wm8804_i2c_devices, ARRAY_SIZE(snd_wm8804_i2c_devices));
 #endif
 
 #if defined(CONFIG_SND_BCM2708_SOC_RPI_DAC) || defined(CONFIG_SND_BCM2708_SOC_RPI_DAC_MODULE)
-        bcm_register_device(&snd_rpi_dac_device);
-        bcm_register_device(&snd_pcm1794a_codec_device);
+	bcm_register_device(&snd_rpi_dac_device);
+	bcm_register_device(&snd_pcm1794a_codec_device);
+#endif
+
+
+#ifdef CONFIG_SND_BCM2708_SOC_RPI_CODEC_MBED_MODULE
+	bcm_register_device(&snd_rpi_mbed_device);
+	i2c_register_board_info(1, snd_rpi_mbed_i2c_devices, ARRAY_SIZE(snd_rpi_mbed_i2c_devices));
+#endif
+
+#ifdef CONFIG_SND_BCM2708_SOC_RPI_CODEC_CS534X_MODULE
+	bcm_register_device(&snd_rpi_cs534x_device);
+	bcm_register_device(&snd_rpi_cs534x_codec_device);
+#endif
+
+#ifdef CONFIG_SND_BCM2708_SOC_RPI_CODEC_TDA1541A_MODULE
+	bcm_register_device(&snd_rpi_tda1541a_device);
+	bcm_register_device(&snd_rpi_tda1541a_codec_device);
+#endif
+
+#ifdef CONFIG_SND_BCM2708_SOC_RPI_CODEC_PROTO_MODULE
+	bcm_register_device(&snd_rpi_proto_device);
+	i2c_register_board_info(1, snd_rpi_proto_i2c_devices, ARRAY_SIZE(snd_rpi_proto_i2c_devices));
+#endif
+
+#ifdef CONFIG_SND_BCM2708_SOC_RPI_CODEC_ESS9018_MODULE
+	bcm_register_device(&snd_rpi_ess9018_device);
+	bcm_register_device(&snd_rpi_ess9018_codec_device);
+#endif
+
+#ifdef CONFIG_SND_BCM2708_SOC_RPI_CODEC_PCM5102A_MODULE
+	bcm_register_device(&snd_rpi_pcm5102a_device);
+	bcm_register_device(&snd_rpi_pcm5102a_codec_device);
 #endif
 
 	for (i = 0; i < ARRAY_SIZE(amba_devs); i++) {
@@ -943,7 +1083,9 @@ static struct platform_device bcm2708_led_device = {
 
 static void __init bcm2708_init_led(void)
 {
-	platform_device_register(&bcm2708_led_device);
+  bcm2708_leds[0].gpio = disk_led_gpio;
+  bcm2708_leds[0].active_low = disk_led_active_low;
+  platform_device_register(&bcm2708_led_device);
 }
 #else
 static inline void bcm2708_init_led(void)
@@ -982,4 +1124,6 @@ MACHINE_END
 module_param(boardrev, uint, 0644);
 module_param(serial, uint, 0644);
 module_param(uart_clock, uint, 0644);
+module_param(disk_led_gpio, uint, 0644);
+module_param(disk_led_active_low, uint, 0644);
 module_param(reboot_part, uint, 0644);
